package todo.app.kotlin.portlet.portlet

import todo.app.kotlin.portlet.constants.TodoWebKotlinPortletKeys

import com.liferay.portal.kernel.portlet.bridges.mvc.MVCPortlet
import com.liferay.portal.kernel.servlet.SessionMessages
import com.liferay.portal.kernel.theme.ThemeDisplay
import com.liferay.portal.kernel.util.ParamUtil
import com.liferay.portal.kernel.util.WebKeys
import com.liferay.todo.service.TodoService

import org.osgi.service.component.annotations.Component
import org.osgi.service.component.annotations.Reference
import java.io.IOException
import javax.portlet.*

/**
 * @author arnaud
 */
@Component(
        immediate = true,
        property = arrayOf(
                    "com.liferay.portlet.display-category=category.sample",
                    "com.liferay.portlet.instanceable=true",
                    "javax.portlet.display-name=Todo List Portlet in Kotlin",
                    "javax.portlet.init-param.template-path=/",
                    "javax.portlet.init-param.view-template=/view.jsp",
                    "javax.portlet.name=" + TodoWebKotlinPortletKeys.TodoWebKotlin,
                    "javax.portlet.resource-bundle=content.Language",
                    "javax.portlet.security-role-ref=power-user,user"
        ),
        service = arrayOf(Portlet::class))

class TodoWebKotlinPortlet : MVCPortlet() {

    @Throws(IOException::class, PortletException::class)
    override fun doView(request: RenderRequest?, response: RenderResponse?) {

        val td = request!!.getAttribute(WebKeys.THEME_DISPLAY) as ThemeDisplay

        request.setAttribute(
                "todoList",
                _todoService!!.todos)

        super.doView(request, response)
    }

    @Throws(IOException::class, PortletException::class)
    override fun processAction(request: ActionRequest, response: ActionResponse) {
        val desc = ParamUtil.getString(request, "task")
        if (!desc.isNullOrEmpty()) {
            SessionMessages.add(request, "todo.successfully.added")
            _todoService!!.addTodo(ParamUtil.getString(request, "task"))
        }
    }

    @Reference
    @Volatile
    lateinit var _todoService : TodoService;

}